#
# MAD Nuke Mod
# CvMADNukeEventManager
# 

from CvPythonExtensions import *

import CvUtil
import CvEventManager
import sys
import PyHelpers
import CvConfigParser
import math
import time
			
gc = CyGlobalContext()	

PyPlayer = PyHelpers.PyPlayer
PyGame = PyHelpers.PyGame()

localText = CyTranslator()

# Change this value to false if nuke strikes should not completely destroy 
# cities.
# Default value is true
g_bNukesDestroyCities = true

# Change this value to false if nuke strikes should not change the plot that is
# hit into an ocean plot. When this value is set to true then the 
# g_bNukesDestroyUnits, g_bNukeStrikesLeaveRuins and g_bNukesDestroyCities are 
# automatically assumed to be true.
# Default value is true
g_bNukesAlterLandscape = true

# Change this value to false if the vanilla unit nuke strike survival code
# should be used instead of completely wiping them out when a nuke strikes a
# city.
# Default value is true
g_bNukesDestroyUnits = true

# Change this value to false if nuke strikes should leave city ruins.
# Default value is true
g_bNukeStrikesDestroyRuins = true

class CvMADNukesEventManager:
	
	def __init__(self, eventManager):

		eventManager.addEventHandler("unitLost", self.onUnitLost)
				
		global g_bNukesDestroyCities
		global g_bNukesAlterLandscape
		global g_bNukesDestroyUnits
		global g_bNukeStrikesDestroyRuins

		config = CvConfigParser.CvConfigParser("MAD Nukes Mod Config.ini")

		if(config != None):
			g_bNukesDestroyCities = config.getboolean("MAD Nukes Mod", "Nukes Destroy Cities", True)
			g_bNukesAlterLandscape = config.getboolean("MAD Nukes Mod", "Nukes Alter Landscape", True)
			g_bNukesDestroyUnits = config.getboolean("MAD Nukes Mod", "Nukes Destroy Units", True)
			g_bNukeStrikesDestroyRuins = config.getboolean("MAD Nukes Mod", "Nuke Strikes Destroy Ruins", True)


	def onUnitLost(self, argsList):
		'Unit Lost'
		objUnit = argsList[0]

		# Only handle the cases where if the unit lost is a nuke
		if(objUnit.getUnitType() == gc.getInfoTypeForString("UNIT_ICBM")):
			self.handleNukeDetonation(objUnit)
		
	
	# This method handles the actual detonation of a nuke. It will basically
	# go through a player's enemy list. For each enemy the method will go
	# through each one of their cities and see if the number of plots with 
	# fallout around a city cross a threshold and perform the appropriate
	# action.
	def handleNukeDetonation(self, objNuke):
	
		# Return immediately if the nuke passed in is invalid
		if(objNuke == None):
			return

		# Return immediately if the nuke passed in is invalid
		if(objNuke.isNone()):
			return

		# Return immediately if the nuke passed in isn't really a nuke.
		if(objNuke.getUnitType() != gc.getInfoTypeForString("UNIT_ICBM")):
			return
		
		# Get the owner of the nuke.	
		iPlayer = objNuke.getOwner()

		# Get the enemies of the player
		enemyList = self.getPlayerEnemies(iPlayer)

		# Go through all the enemies in the list
		for objPlayer in enemyList:
			
			# Get all of the enemies cities
			cityList = objPlayer.getCityList()

			# Go through all of the enemies cities
			for objCity in cityList:
				
				# Get the CyCity instance of the city
				objCity = objCity.GetCy()
				falloutPlotList = []
				iTotalNonWaterPlots = -1
				
				# Go through plots adjacent to the city
				for i in range(9):
				
					# Save the plot if it cannot be worked by the city and it 
					# contains fallout
					if(not objCity.canWork(objCity.getCityIndexPlot(i)) and objCity.getCityIndexPlot(i).getFeatureType() == gc.getInfoTypeForString("FEATURE_FALLOUT")):
						falloutPlotList.append(objCity.getCityIndexPlot(i))

					# Increment the iTotalNonWaterPlots counter if the plot is 
					# water
					if(not objCity.getCityIndexPlot(i).isWater()):
						iTotalNonWaterPlots+=1
				
				# Calculate the fallout threshold	
				iFalloutThreshold = iTotalNonWaterPlots/2

				# If the number of plots with fallout exceed the fallout 
				# threshold then perform the appropriate action
				if(len(falloutPlotList) >= iFalloutThreshold):
					# Get the plot the city is on
					objCityPlot = objCity.plot()

					if(g_bNukesAlterLandscape):
						self.devistatePlot(objCity, iPlayer)
						continue

					# Destroy the route that was on the same plot as the city
					objCityPlot.setRouteType(gc.getInfoTypeForString("NO_ROUTE"))
						
					# Go through all of the units in the city and kill them off 
					# if the mod is configured to do so
					if(g_bNukesDestroyUnits):
						for i in range(objCityPlot.getNumUnits()):
							objCityPlot.getUnit(i).kill(true,iPlayer)
					
					# Kill the city	if mod is configured to do so
					if(g_bNukesDestroyCities):
						objCity.kill()
					
					# Destroy the improvement that was on the same plot as the 
					# city if mod is configured to do so
					if(g_bNukeStrikesDestroyRuins):
						objCityPlot.setImprovementType(gc.getInfoTypeForString("NO_IMPROVEMENTS"))

		# Refresh and redraw the interface
		CyInterface().setDirty(InterfaceDirtyBits.MinimapSection_DIRTY_BIT, true)
		CyInterface().setDirty(InterfaceDirtyBits.MiscButtons_DIRTY_BIT, true)


	# Completely devistates the plot that the city is on and turns it into an
	# ocean plot.
	def devistatePlot(self, objCity, iPlayer):

		# Return immediately if the city passed in is invalid
		if(objCity == None):
			return

		# Return immediately if the city passed in is invalid
		if(objCity.isNone()):
			return
		
		# Get the plot the city is on
		objCityPlot = objCity.plot()

		# Return an empty list immediately if the player passed in is invalid
		if(iPlayer < 0):
			return 

		objPlayer = gc.getPlayer(iPlayer)

		# Return an empty list immediately if the player passed in is invalid
		if(objPlayer == None):
			return 

		# Return an empty list immediately if the player passed in is invalid
		if(objPlayer.isNone()):
			return 

		# Go through all of the units in the city and kill them off
		for i in range(objCityPlot.getNumUnits()):
			objCityPlot.getUnit(i).kill(true,iPlayer)

		# Kill the city					
		objCity.kill()

		# Destroy the improvement that was on the same plot as the city
		objCityPlot.setImprovementType(gc.getInfoTypeForString("NO_IMPROVEMENTS"))

		# Destroy the route that was on the same plot as the city
		objCityPlot.setRouteType(gc.getInfoTypeForString("NO_ROUTE"))
		
		# Change the plot type of the plot where the city was
		objCityPlot.setPlotType(PlotTypes(gc.getInfoTypeForString("PLOT_OCEAN")), True, True)
		
		# Change the terrain type of the plot where the city was
		objCityPlot.setTerrainType(gc.getInfoTypeForString("TERRAIN_OCEAN"), True, True)

		# Refresh and redraw the interface
		CyInterface().setDirty(InterfaceDirtyBits.MinimapSection_DIRTY_BIT, true)
		CyInterface().setDirty(InterfaceDirtyBits.MiscButtons_DIRTY_BIT, true)			
	
	
	# Returns all of the enemies for the player passed in.
	def getPlayerEnemies(self, iPlayer):
	
		# Return an empty list immediately if the player passed in is invalid
		if(iPlayer < 0):
			return []

		objPlayer = gc.getPlayer(iPlayer)

		# Return an empty list immediately if the player passed in is invalid
		if(objPlayer == None):
			return []

		# Return an empty list immediately if the player passed in is invalid
		if(objPlayer.isNone()):
			return []
			
		# Get the active players in the game
		activePlayerList = PyGame.getCivPlayerList()
		
		enemyList = []

		# Get the team for the player passed in
		objPlayerTeam = gc.getTeam(objPlayer.getTeam())
			
		# Go through all of the players active in the game
		for objTempPlayer in activePlayerList:

			# Continue if the current player is the same as the player passed 
			# in.
			if(iPlayer == objTempPlayer.getID()):
				continue
			
			# Add the current player to the enemy list if the current player is
			# an enemy of the player passed in	
			if(objPlayerTeam.isAtWar(objTempPlayer.CyGet().getTeam())):
				enemyList.append(objTempPlayer)
				
		return enemyList				
		